@@ -14,28 +14,43 @@
 ##' A \code{character} corresponding to the modeling folder
 ##' @param resp.name a \code{character} corresponding to the species name
 ##' 
-##' @param resp.var a \code{vector}, \code{\link[sp]{SpatialPoints}} (\emph{if presence-only}) or 
-##' \code{\link[sp]{SpatialPointsDataFrame}} object containing binary data (\code{0} : absence, 
-##' \code{1} : presence, \code{NA} : indeterminate) for a single species that will be used to 
-##' build the species distribution model(s)
-##' @param expl.var a \code{matrix}, \code{data.frame}, \code{\link[sp]{SpatialPointsDataFrame}} 
-##' or \code{\link[raster:stack]{RasterStack}} object containing the explanatory variables 
+##' @param resp.var a \code{vector}, a \code{\link[terra:vect]{SpatVector}}
+##' without associated data (\emph{if presence-only}), 
+##' or a \code{\link[terra:vect]{SpatVector}} object containing binary data  
+##' (\code{0} : absence, \code{1} : presence, \code{NA} : indeterminate) 
+##' for a single species that will be used to build the species distribution model(s)
+##' \cr \emph{Note that old format from \pkg{sp} are still supported such as
+##'  \code{SpatialPoints}  (if presence-only) or \code{SpatialPointsDataFrame}
+##'  object containing binary data.}
+##' 
+##' @param expl.var a \code{matrix}, \code{data.frame}, \code{\link[terra:vect]{SpatVector}}
+##' or \code{\link[terra:rast]{SpatRaster}} object containing the explanatory variables 
 ##' (in columns or layers) that will be used to build the species distribution model(s)
+##' \cr \emph{Note that old format from \pkg{raster} and \pkg{sp} are still supported such as 
+##' \code{RasterStack} and \code{SpatialPointsDataFrame} objects. }
+##' 
 ##' @param resp.xy (\emph{optional, default} \code{NULL}) \cr 
 ##' If \code{resp.var} is a \code{vector}, a 2-columns \code{matrix} or \code{data.frame} 
-##' containing the corresponding \code{X} and \code{Y} coordinates that will be used to build the 
-##' species distribution model(s)
+##' containing the corresponding \code{X} and \code{Y} coordinates that will be
+##' used to build the species distribution model(s)
 ##' 
 ##' @param eval.resp.var (\emph{optional, default} \code{NULL}) \cr 
-##' A \code{vector}, \code{\link[sp]{SpatialPoints}} (\emph{if presence-only}) or 
-##' \code{\link[sp]{SpatialPointsDataFrame}} object containing binary data (\code{0} : absence, 
-##' \code{1} : presence, \code{NA} : indeterminate) for a single species that will be used to 
+##' A \code{vector}, a \code{\link[terra:vect]{SpatVector}}
+##' without associated data (\emph{if presence-only}), 
+##' or a \code{\link[terra:vect]{SpatVector}} object containing binary data  
+##' (\code{0} : absence, \code{1} : presence, \code{NA} : indeterminate) 
+##'  for a single species that will be used to 
 ##' evaluate the species distribution model(s) with independent data
+##' \cr \emph{Note that old format from \pkg{sp} are still supported such as
+##'  \code{SpatialPoints}  (if presence-only) or \code{SpatialPointsDataFrame}
+##'  object containing binary data.}
 ##' @param eval.expl.var (\emph{optional, default} \code{NULL}) \cr 
-##' A \code{matrix}, \code{data.frame}, \code{\link[sp]{SpatialPointsDataFrame}} or 
-##' \code{\link[raster:stack]{RasterStack}} object containing the explanatory variables (in 
+##' A \code{matrix}, \code{data.frame}, \code{\link[terra:vect]{SpatVector}}
+##' or \code{\link[terra:rast]{SpatRaster}} object containing the explanatory variables (in 
 ##' columns or layers) that will be used to evaluate the species distribution model(s) with 
-##' independent data
+##' independent data.
+##' \cr \emph{Note that old format from \pkg{raster} and \pkg{sp} are still supported such as 
+##' \code{RasterStack} and \code{SpatialPointsDataFrame} objects. }
 ##' @param eval.resp.xy (\emph{optional, default} \code{NULL}) \cr 
 ##' If \code{resp.var} is a \code{vector}, a 2-columns \code{matrix} or \code{data.frame} 
 ##' containing the corresponding \code{X} and \code{Y} coordinates that will be used to evaluate 
@@ -73,34 +88,41 @@
 ##' @param na.rm (\emph{optional, default} \code{TRUE}) \cr 
 ##' A \code{logical} value defining whether points having one or several missing values for 
 ##' explanatory variables should be removed from the analysis or not
+##' @param filter.raster (\emph{optional, default} \code{FALSE}) \cr 
+##' If \code{expl.var} is of raster type, a \code{logical} value defining whether \code{resp.var} 
+##' is to be filtered when several points occur in the same raster cell
 ##' 
 ##' 
 ##' @return 
 ##' 
 ##' A \code{BIOMOD.formated.data} object that can be used to build species distribution model(s) 
-##' with the \code{\link{BIOMOD_Modeling}} function. \cr \code{print} and \code{plot} 
-##' functions are available to have a summary of the created object.
+##' with the \code{\link{BIOMOD_Modeling}} function. \cr
+##' \code{print} and \code{plot} functions are available to have a summary of the created object.
 ##' 
 ##' 
 ##' @details  
 ##' 
 ##' This function gathers and formats all input data needed to run \pkg{biomod2} models. It 
-##' supports different kind of inputs (e.g. \code{matrix}, \code{\link[sp]{SpatialPoints}}, 
-##' \code{\link[raster:stack]{RasterStack}}) and provides different methods to select 
-##' pseudo-absences if needed. \cr \cr
+##' supports different kind of inputs (e.g. \code{matrix},
+##' \code{\link[terra:vect]{SpatVector}}, \code{\link[terra:rast]{SpatRaster}})
+##' and provides different methods to select pseudo-absences if needed. \cr \cr
 ##' 
 ##' \bold{Concerning explanatory variables and XY coordinates :} 
 ##' \itemize{
-##'   \item if \code{\link[raster:stack]{RasterLayer}} or \code{\link[raster:stack]{RasterStack}} 
-##'   provided (for \code{expl.var} or \code{eval.expl.var}), \pkg{biomod2} will extract the 
-##'   corresponding values from XY coordinates provided (\code{resp.xy} or \code{eval.resp.xy} 
-##'   respectively). \cr \emph{Be sure to give the XY coordinates in the same projection system 
-##'   than the raster objects !}
-##'   \item if \code{\link[sp]{SpatialPointsDataFrame}} provided (for \code{resp.var} or 
-##'   \code{eval.resp.var}), the same rule applies (\emph{same projection system !}).
-##'   \item if \code{data.frame} provided (for \code{expl.var} or \code{eval.expl.var}), 
-##'   \pkg{biomod2} will simply merge it (\code{cbind}) with \code{resp.var} without 
-##'   considering XY coordinates. \cr
+##'   \item if \code{\link[terra:rast]{SpatRaster}}, \code{RasterLayer} or \code{RasterStack}
+##'   provided for \code{expl.var} or \code{eval.expl.var}, \cr \pkg{biomod2} will extract 
+##'   the corresponding values from XY coordinates provided :
+##'   \itemize{
+##'     \item either through \code{resp.xy} or \code{eval.resp.xy} respectively
+##'     \item or \code{resp.var} or \code{eval.resp.var}, if provided as 
+##'     \code{\link[terra:vect]{SpatVector}} or \code{SpatialPointsDataFrame}
+##'   }
+##'   \emph{Be sure to give the objects containing XY coordinates in the same projection 
+##'   system than the raster objects !}
+##'    
+##'   \item if \code{data.frame} or \code{matrix} provided for \code{expl.var} or
+##'    \code{eval.expl.var}, \cr \pkg{biomod2} will simply merge it (\code{cbind}) 
+##'    with \code{resp.var} without considering XY coordinates. \cr
 ##'   \emph{Be sure to give explanatory and response values in the same row order !}
 ##' }
 ##' 
@@ -121,8 +143,10 @@
 ##'   \item{Response variable}{
 ##'   \pkg{biomod2} models single species at a time (no multi-species). Hence, \code{resp.var} 
 ##'   must be a uni-dimensional object (either a \code{vector}, a one-column \code{matrix}, 
-##'   \code{data.frame} or \code{\link[sp]{SpatialPointsDataFrame}}, or a 
-##'   \code{\link[sp]{SpatialPoints}} object if presence-only), containing values among :
+##'   \code{data.frame}, a \code{\link[terra:vect]{SpatVector}} (\emph{without associated 
+##'   data - if presence-only}), a \code{SpatialPoints} (\emph{if presence-only}), 
+##'   a \code{SpatialPointsDataFrame} or \code{\link[terra:vect]{SpatVector}} object),
+##'    containing values among :
 ##'   \itemize{
 ##'     \item \code{1} : presences
 ##'     \item \code{0} : true absences (if any)
@@ -140,18 +164,18 @@
 ##'   }
 ##'   \item{Evaluation data}{
 ##'   Although \pkg{biomod2} provides tools to automatically divide dataset into calibration and 
-##'   validation parts through the modeling process (see \code{NbRunEval} and \code{DataSplit} 
+##'   validation parts through the modeling process (see \code{nb.rep} and \code{data.split.perc} 
 ##'   parameters in \code{\link{BIOMOD_Modeling}} function ; or \code{\link{BIOMOD_CrossValidation} 
 ##'   function}), it is also possible (and strongly advised) to directly provide two independent 
-##'   datasets, one for calibration and one for validation.
+##'   datasets, one for calibration/validation and one for evaluation
 ##'   }
 ##'   \item{Pseudo-absence selection}{
 ##'   If no true absences are available, pseudo-absences must be selected from the 
 ##'   \emph{background data}, meaning data there is no information whether the species of 
 ##'   interest occurs or not. It corresponds either to the remaining pixels of the \code{expl.var} 
-##'   (if provided as a \code{\link[raster:stack]{RasterStack}}) or to the points identified as 
-##'   \code{NA} in \code{resp.var} (if \code{expl.var} provided as a \code{matrix} or 
-##'   \code{data.frame}). \cr 
+##'   (if provided as a \code{\link[terra:rast]{SpatRaster}} or \code{RasterSatck})
+##'   or to the points identified as  \code{NA} in \code{resp.var} (if \code{expl.var}
+##'   provided as a \code{matrix} or \code{data.frame}). \cr 
 ##'   Several methods are available to do this selection :
 ##'   \describe{
 ##'     \item{random}{all points of initial background are pseudo-absence candidates. 
@@ -185,10 +209,10 @@
 ##' 
 ##' 
 ##' @examples
+##' library(terra)
 ##' 
 ##' # Load species occurrences (6 species available)
-##' myFile <- system.file('external/species/mammals_table.csv', package = 'biomod2')
-##' DataSpecies <- read.csv(myFile, row.names = 1)
+##' data(DataSpecies)
 ##' head(DataSpecies)
 ##' 
 ##' # Select the name of the studied species
@@ -201,25 +225,26 @@
 ##' myRespXY <- DataSpecies[, c('X_WGS84', 'Y_WGS84')]
 ##' 
 ##' # Load environmental variables extracted from BIOCLIM (bio_3, bio_4, bio_7, bio_11 & bio_12)
-##' myFiles <- paste0('external/bioclim/current/bio', c(3, 4, 7, 11, 12), '.grd')
-##' myExpl <- raster::stack(system.file(myFiles, package = 'biomod2'))
+##' data(bioclim_current)
+##' myExpl <- terra::rast(bioclim_current)
 ##' 
 ##' \dontshow{
-##' myExtent <- raster::extent(0,30,45,70)
-##' myExpl <- raster::stack(raster::crop(myExpl, myExtent))
+##' myExtent <- terra::ext(0,30,45,70)
+##' myExpl <- terra::crop(myExpl, myExtent)
 ##' }
 ##' 
-##' # ---------------------------------------------------------------
+##' # ---------------------------------------------------------------#
 ##' # Format Data with true absences
 ##' myBiomodData <- BIOMOD_FormatingData(resp.var = myResp,
 ##'                                      expl.var = myExpl,
 ##'                                      resp.xy = myRespXY,
 ##'                                      resp.name = myRespName)
 ##' myBiomodData
+##' summary(myBiomodData)
 ##' plot(myBiomodData)
 ##' 
 ##' 
-##' # ---------------------------------------------------------------
+##' # ---------------------------------------------------------------#
 ##' # # Transform true absences into potential pseudo-absences
 ##' # myResp.PA <- ifelse(myResp == 1, 1, NA)
 ##' # 
@@ -274,11 +299,11 @@
 ##' # plot(myBiomodData.u)
 ##' 
 ##' 
-##' @importFrom sp coordinates
-##' @importFrom raster stack
+##' @importFrom terra rast crds
 ##' 
 ##' @export
 ##' 
+##' 
 ###################################################################################################
 
 BIOMOD_FormatingData <- function(resp.name,
@@ -296,7 +321,8 @@ BIOMOD_FormatingData <- function(resp.name,
                                  PA.dist.max = NULL,
                                  PA.sre.quant = 0.025,
                                  PA.user.table = NULL,
-                                 na.rm = TRUE)
+                                 na.rm = TRUE,
+                                 filter.raster = FALSE)
 {
   .bm_cat(paste0(resp.name, " Data Formating"))
   
@@ -315,10 +341,10 @@ BIOMOD_FormatingData <- function(resp.name,
                                            PA.dist.min,
                                            PA.dist.max,
                                            PA.sre.quant,
-                                           PA.user.table)
+                                           PA.user.table,
+                                           filter.raster)
   for (argi in names(args)) { assign(x = argi, value = args[[argi]]) }
   rm(args)
-  
   ## 2. build BIOMOD.formated.data object -------------------------------------
   out <- NULL
   if( PA.strategy == 'none') { # no Pseudo Absences
@@ -330,7 +356,8 @@ BIOMOD_FormatingData <- function(resp.name,
                                 eval.sp = eval.resp.var,
                                 eval.env = eval.expl.var,
                                 eval.xy = eval.resp.xy,
-                                na.rm = na.rm)
+                                na.rm = na.rm,
+                                filter.raster = filter.raster)
   } else { # automatic Pseudo Absences selection
     out <- BIOMOD.formated.data.PA(sp = resp.var,
                                    xy = resp.xy,
@@ -347,14 +374,15 @@ BIOMOD_FormatingData <- function(resp.name,
                                    PA.dist.max = PA.dist.max,
                                    PA.sre.quant = PA.sre.quant,
                                    PA.user.table = PA.user.table,
-                                   na.rm = na.rm)
+                                   na.rm = na.rm,
+                                   filter.raster = filter.raster)
   }
   .bm_cat("Done")
   return(out)
 }
 
 
-###################################################################################################
+# Argument Check -------------------------------------------------------------
 
 .BIOMOD_FormatingData.check.args <- function(resp.name,
                                              resp.var,
@@ -370,7 +398,8 @@ BIOMOD_FormatingData <- function(resp.name,
                                              PA.dist.min,
                                              PA.dist.max,
                                              PA.sre.quant,
-                                             PA.user.table)
+                                             PA.user.table,
+                                             filter.raster)
 {
   ## 0. Checking names (resp.name available ?) --------------------------------
   if (grepl('/', resp.name)) {
@@ -386,94 +415,86 @@ BIOMOD_FormatingData <- function(resp.name,
     stop(paste0("Modeling folder '", dir.name, "' does not exist"))
   }
   
-  available.types <- c('integer', 'numeric', 'data.frame', 'matrix',
-                       'RasterLayer', 'RasterStack',
-                       'SpatialPointsDataFrame', 'SpatialPoints')
-  
-  ## 1. Checking resp.var -----------------------------------------------------
-  
-  if (!inherits(resp.var, available.types)) { ## resp.var
-    stop(paste0("Response variable must be one of ", toString(available.types)))
-  }
-  
-  if (inherits(resp.var, 'Raster')) { ## resp.var raster object not supported yet
+  if (inherits(resp.var, c('Raster','SpatRaster'))) { 
+    ## resp.var raster object not supported yet
     stop("Raster response variable not supported yet ! \nPlease extract your Presences and your absences by yourself")
     #### TO DO #### extract the 0 and 1 in sp format
   }
   
-  if (!inherits(expl.var, setdiff(available.types, 'SpatialPoints'))) { ## expl.var
-    stop(paste0("Explanatory variable must be one of ", toString(available.types)))
-  }
   
-  if (inherits(resp.var, 'SpatialPoints')) { ## resp.xy
-    if (!is.null(resp.xy)) {
-      cat("\n      ! XY coordinates of response variable will be ignored because spatial response object is given.")
-    }
-    resp.xy <- data.matrix(coordinates(resp.var))
-    if (inherits(resp.var, 'SpatialPointsDataFrame')) {
-      resp.var <- resp.var@data
-    } else {
-      cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
-      resp.var <- rep(1, nrow(resp.xy))
-    }
-  }
+  ## 1. Checking resp.var and expl.var -----------------------------------------
   
-  ## transforming into numeric if data.frame or matrix
-  if (is.matrix(resp.var) | is.data.frame(resp.var)) {
-    resp.var = as.data.frame(resp.var)
-    if (ncol(resp.var) > 1) {
-      stop("You must give a monospecific response variable (1D object)")
-    } else {
-      resp.var <- as.numeric(resp.var[, 1])
-    }
-  }
   
-  if(is.matrix(expl.var) | is.numeric(expl.var)) {
-    expl.var <- as.data.frame(expl.var)
-  }
+  ### 1.1 Checking resp.var ----------------------------------------------------
+  available.types.resp <- c('integer', 'numeric', 'data.frame', 'matrix',
+                            'SpatialPointsDataFrame', 'SpatialPoints', 'SpatVector')
   
-  if (inherits(expl.var, 'Raster')) {
-    expl.var <- stack(expl.var, RAT = FALSE)
-  }
+  ####  Check response type -----------------------------------------------
   
-  if (inherits(expl.var, 'SpatialPoints')) {
-    expl.var <- as.data.frame(expl.var@data)
+  .fun_testIfInherits(TRUE, "resp.var", resp.var, available.types.resp)
+  
+  ####  SpatialPoints, SpatialPointsDataFrame and SpatVector -------------------
+  if (inherits(resp.var, c('SpatialPoints','SpatVector'))) { 
+    .tmp <- .check_formating_spatial(resp.var = resp.var,
+                                     expl.var = expl.var, 
+                                     resp.xy = resp.xy,
+                                     eval.data = FALSE)
+    resp.var <- .tmp$resp.var
+    resp.xy <- .tmp$resp.xy
+    rm(.tmp)
   }
+  ####  data.frame and matrix --------------------------
+  ## transforming into numeric
+  if (inherits(resp.var, c("matrix","data.frame"))) {
+    resp.var <- .check_formating_table(resp.var)
+  }
+  ####  checking xy coordinates validity --------------------------
   
-  ## checking xy coordinates validity
   if(!is.null(resp.xy)){
-    if(ncol(resp.xy)!=2){
-      stop("If given, resp.xy must be a 2 column matrix or data.frame")
-    }
-    if(nrow(resp.xy) != length(resp.var)){
-      stop("Response variable and its coordinates don't match")
-    }
-    resp.xy <- as.data.frame(resp.xy)
+    resp.xy <- .check_formating_xy(resp.xy, 
+                                   resp.length = length(resp.var))
+  } else if (inherits(expl.var, c('RasterLayer', 'RasterStack', 'SpatRaster'))) {
+    stop("`resp.xy` argument is missing. Please provide `resp.xy` when `expl.var` is a raster.")
+  } else {
+    resp.xy <- data.frame()
   }
   
-  ## converting response var into binary
-  resp.var[which(resp.var > 0)] <- 1
-  resp.var[which(resp.var <= 0)] <- 0
+  
+  #### check presence/absence in resp.var -------------------------------------
+  
+  resp.var <- .check_formating_resp.var(resp.var = resp.var, eval.data = FALSE)
+  
+  
+  ### 1.2 checking expl.var -------------------------------------------------------
+  
+  available.types.expl <- c('integer', 'numeric', 'data.frame', 'matrix',
+                            'RasterLayer', 'RasterStack', 'SpatRaster',
+                            'SpatialPointsDataFrame', 'SpatVector')
+  
+  .fun_testIfInherits(TRUE, "expl.var",
+                      expl.var, available.types.expl)
+  
+  expl.var <- .check_formating_expl.var(expl.var ,
+                                        length.resp.var = length(resp.var))
   
   #### At this point :
   ####  - resp.var is a numeric
   ####  - resp.xy is NULL or a data.frame
-  ####  - expl.var is a data.frame or a RasterStack
+  ####  - expl.var is a data.frame or a SpatRaster
   ####  - sp.name is a character
   
-  ## checking resp and expl var compatibility
-  if (is.data.frame(expl.var) && nrow(expl.var) != length(resp.var)) {
-    stop("If explanatory variable is not a raster then dimensions of response variable and explanatory variable must match!")
-  }
   
-  ## PA strategy
-  if (is.null(PA.user.table) & PA.nb.rep < 1) {
+  ## 2. PA strategy ------------------------------------------------------------
+  if (is.null(PA.user.table) && PA.nb.rep < 1) {
+    if (!any(resp.var == 0, na.rm = TRUE) && !any(is.na(resp.var))) {
+      stop("No Absences were given and no Pseudo-Absences were given or configured, at least one of those option is required.")
+    }
     cat("\n> No pseudo absences selection !")
     PA.strategy <- "none"
     PA.nb.rep <- 0
   }
   
-  if (is.null(PA.strategy) &  PA.nb.rep > 0) {
+  if (is.null(PA.strategy) &&  PA.nb.rep > 0) {
     cat("\n> Pseudo absences will be selected randomly !")
     PA.strategy <- "random"
   }
@@ -494,76 +515,66 @@ BIOMOD_FormatingData <- function(resp.name,
     colnames(PA.user.table) <- paste0("PA", 1:ncol(PA.user.table))
   }
   
-  ## 2. Checking eval.resp.var ------------------------------------------------
+  ## 3. Checking eval.resp.var and eval.expl.var -------------------------------
+  
+  ### 3.1 Checking eval.resp.var ---------------------------------------------------
+  available.types.eval.resp <- c('integer', 'numeric', 'data.frame', 'matrix',
+                                 'SpatialPointsDataFrame', 'SpatVector')
   
   if (!is.null(eval.resp.var)) {
-    if (!(class(eval.resp.var)[1] %in% available.types)) { ## eval.respvar
-      stop(paste0("Response variable must be one of ", toString(available.types)))
-    }
-    
-    if (inherits(eval.resp.var, 'Raster')) { ## eval.resp.var raster object not supported yet
+    if (inherits(eval.resp.var, c('Raster', 'SpatRaster'))) {
+      ## eval.resp.var raster object not supported yet
       stop("Raster response variable not supported yet ! \nPlease extract your Presences and your absences by yourself")
       #### TO DO #### extract the 0 and 1 in sp format
     }
     
-    if (!is.null(eval.expl.var)) { ## eval.expl.var
-      if (!(class(eval.expl.var)[1] %in% available.types[-which(available.types == 'SpatialPoints')])) {
-        stop(paste0("Explanatory variable must be one of ", toString(available.types)))
-      }
-    } else {
-      if (!(inherits(expl.var, 'Raster'))) {
+    .fun_testIfInherits(TRUE, "eval.resp.var",
+                        eval.resp.var, available.types.eval.resp)
+    
+    ####  SpatialPoints, SpatialPointsDataFrame and SpatVector -------------------
+    if (inherits(eval.resp.var, c('SpatialPoints','SpatVector'))) { 
+      .tmp <- .check_formating_spatial(resp.var = eval.resp.var,
+                                       expl.var = eval.expl.var, 
+                                       resp.xy = eval.resp.xy,
+                                       eval.data = TRUE)
+      eval.resp.var <- .tmp$resp.var
+      eval.resp.xy <- .tmp$resp.xy
+      rm(.tmp)
+    }
+    ### Matrix and data.frame -------------------------------------------------
+    
+    if (inherits(eval.resp.var, c("matrix","data.frame"))) {
+      eval.resp.var <- .check_formating_table(eval.resp.var)
+    }
+    
+    ### checking xy coordinates validity ----------------------------
+    if(!is.null(eval.resp.xy)){
+      eval.resp.xy <- .check_formating_xy(eval.resp.xy, 
+                                          resp.length = length(eval.resp.var))
+    }
+    
+    ### check presences and absences in evaluation dataset -------------------
+    eval.resp.var <- .check_formating_resp.var(resp.var = eval.resp.var,
+                                               eval.data = TRUE)
+    
+    ## 3.2 Checking eval.expl.var ---------------------------------------------------
+    
+    available.types.eval.expl <- c('integer', 'numeric', 'data.frame', 'matrix',
+                                   'RasterLayer', 'RasterStack', 'SpatRaster',
+                                   'SpatialPointsDataFrame', 'SpatVector', "NULL")
+    
+    .fun_testIfInherits(TRUE, "eval.expl.var",
+                        eval.expl.var, available.types.eval.expl)
+    
+    if (is.null(eval.expl.var)) {
+      if (!(inherits(expl.var, 'SpatRaster'))) {
         stop("If explanatory variable is not a raster and you want to consider evaluation response variable, you have to give evaluation explanatory variables")
       }
     }
     
-    if (inherits(eval.resp.var, 'SpatialPoints')) { ## eval.resp.xy
-      if (!is.null(eval.resp.xy)) {
-        cat("\n      ! XY coordinates of response variable will be ignored because spatial response object is given.")
-      }
-      eval.resp.xy <- data.matrix(coordinates(eval.resp.var))
-      if (class(eval.resp.var)[1] == 'SpatialPointsDataFrame') {
-        eval.resp.var <- eval.resp.var@data
-      } else {
-        cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
-        eval.resp.var <- rep(1, nrow(eval.resp.xy))
-      }
-    }
+    eval.expl.var <- .check_formating_expl.var(eval.expl.var, 
+                                               length.resp.var = length(eval.resp.var))
     
-    ### transforming into numeric if data.frame or matrix
-    if (is.matrix(eval.resp.var) | is.data.frame(eval.resp.var)) {
-      if (ncol(eval.resp.var) > 1) {
-        stop("You must give a monospecific response variable (1D object)")
-      } else {
-        eval.resp.var <- as.numeric(eval.resp.var[, 1])
-      }
-    }
-    
-    if (is.matrix(eval.expl.var) | is.numeric(eval.expl.var)) {
-      eval.expl.var <- as.data.frame(eval.expl.var)
-    }
-    
-    if (inherits(eval.expl.var, 'Raster')) {
-      eval.expl.var <- stack(eval.expl.var)
-    }
-    
-    if (inherits(eval.expl.var, 'SpatialPoints')) {
-      eval.expl.var <- as.data.frame(eval.expl.var@data)
-    }
-    
-    ## checking xy coordinates validity
-    if (!is.null(eval.resp.xy)) {
-      if (ncol(eval.resp.xy) != 2) {
-        stop("if given, resp.xy must be a 2 column matrix or data.frame")
-      }
-      if (nrow(eval.resp.xy) != length(eval.resp.var)) {
-        stop("Response variable and its coordinates don't match")
-      }
-      eval.resp.xy <- as.data.frame(eval.resp.xy)
-    }
-    
-    if (is.data.frame(eval.expl.var) && nrow(eval.expl.var) != length(eval.resp.var)) {
-      stop("If explanatory variable is not a raster then dimensions of response variable and explanatory variable must match!")
-    }
     
     ## remove NA from evaluation data
     if (sum(is.na(eval.resp.var)) > 0) {
@@ -574,19 +585,18 @@ BIOMOD_FormatingData <- function(resp.name,
       eval.resp.var <- na.omit(eval.resp.var)
     }
     
-    ## converting response var into binary
-    eval.resp.var[which(eval.resp.var > 0)] <- 1
-    eval.resp.var[which(eval.resp.var <= 0)] <- 0
-    
-    ## check there are both presences and absences in evaluation dataset
-    if (sum(eval.resp.var == 1) < 1 | sum(eval.resp.var == 0) < 1) {
-      stop("Evaluation response data must have both presences and absences")
-    }
   } else {
     cat("\n      ! No data has been set aside for modeling evaluation")
     eval.expl.var <- eval.resp.xy <- NULL
   }
   
+  
+  
+  ### 4 - argument filter.raster ---------------------------------------------------
+  if(inherits(expl.var, "SpatRaster")){
+    stopifnot(is.logical(filter.raster))
+  }
+  
   ### PA arguments are not checked here because it will be done later... (may be will do it here later)
   
   return(list(resp.var = resp.var,
@@ -605,3 +615,129 @@ BIOMOD_FormatingData <- function(resp.name,
               PA.sre.quant = PA.sre.quant,
               PA.user.table = PA.user.table))
 }
+
+
+# Common tools ------------------------------------------------------------
+
+
+
+.check_formating_spatial <- function(resp.var, expl.var = NULL, resp.xy = NULL, eval.data = FALSE){
+  if (!is.null(resp.xy)) {
+    cat("\n      ! XY coordinates of response variable will be ignored because spatial response object is given.")
+  }
+  
+  if (inherits(resp.var, 'SpatialPoints')) { 
+    resp.xy <- data.matrix(sp::coordinates(resp.var))
+    if (inherits(resp.var, 'SpatialPointsDataFrame')) {
+      resp.var <- resp.var@data
+    } else {
+      cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
+      resp.var <- rep(1, nrow(resp.xy))
+    }
+  }
+  
+  if (inherits(resp.var, 'SpatVector')) { 
+    resp.xy <- data.matrix(crds(resp.var))
+    resp.var <- as.data.frame(resp.var)
+    if (ncol(resp.var) == 0) {
+      if(eval.data){
+        stop("eval.resp must have both presences and absences in the data associated to the SpatVector") 
+      } else {
+        cat("\n      ! Response variable is considered as only presences... Is it really what you want?")
+        resp.var <- rep(1, nrow(resp.xy))
+      }
+    }
+  }
+  
+  if(!eval.data){
+    if ( all(!is.na(resp.var)) && 
+         all(resp.var == 1, na.rm = TRUE) &&
+         !inherits(expl.var, c('Raster','SpatRaster'))) {
+      stop("For Presence-Only model based on SpatialPoints or SpatVector, expl.var needs to be a RasterStack or SpatRaster to be able to sample pseudo-absences")
+    }
+  }
+  
+  return(
+    list(resp.var = resp.var,
+         resp.xy = resp.xy)
+  )
+}
+
+.check_formating_resp.var <- function(resp.var, eval.data = FALSE){
+  
+  if (any(!resp.var %in% c(0,1,NA))) {
+    cat("\n      ! ", ifelse(eval.data, "Evaluation",""), "Response variable have non-binary values that will be converted into 0 (resp <=0) or 1 (resp > 0).")
+    resp.var[which(resp.var > 0)] <- 1
+    resp.var[which(resp.var <= 0)] <- 0
+  }
+  
+  if (eval.data) {
+    if (!any(resp.var == 1, na.rm = TRUE) || !any(resp.var == 0, na.rm = TRUE))
+    {
+      stop("Evaluation response data must have both presences and absences")
+    }
+  }
+  
+  resp.var
+}
+
+.check_formating_table <- function(resp.var){
+  resp.var = as.data.frame(resp.var)
+  if (ncol(resp.var) > 1) {
+    stop("You must give a monospecific response variable (1D object)")
+  } else {
+    resp.var <- as.numeric(resp.var[, 1])
+  }
+  resp.var
+}
+
+.check_formating_xy <- function(resp.xy, resp.length){
+  if (ncol(resp.xy) != 2) {
+    stop("If given, resp.xy must be a 2 column matrix or data.frame")
+  }
+  if (nrow(resp.xy) != resp.length) {
+    stop("Response variable and its coordinates don't match")
+  }
+  as.data.frame(resp.xy)
+}
+
+.check_formating_expl.var <- function(expl.var, length.resp.var){
+  
+  if (is.matrix(expl.var) | is.numeric(expl.var)) {
+    expl.var <- as.data.frame(expl.var)
+  }
+  
+  if (inherits(expl.var, 'Raster')) {
+    expl.var <- raster::stack(expl.var, RAT = FALSE)
+    if (any(is.factor(expl.var))) {
+      expl.var <- categorical_stack_to_terra(expl.var)
+    } else {
+      # as of 20/10/2022 the line below does not work if categorical variables
+      # are present, hence the trick above. 
+      expl.var <- rast(expl.var)
+    }
+  }
+  if (inherits(expl.var, 'SpatialPoints')) {
+    expl.var <- as.data.frame(expl.var@data)
+  }
+  if (inherits(expl.var, 'SpatVector')) {
+    expl.var <- as.data.frame(expl.var)
+  }
+  
+  if(inherits(expl.var, 'data.frame')){
+    if (nrow(expl.var) != length.resp.var) {
+      stop("If explanatory variable is not a raster then dimensions of response variable and explanatory variable must match!")
+    }
+  }
+  expl.var
+}
+
+categorical_stack_to_terra <- function(myraster){
+  rast(
+    sapply(1:raster::nlayers(myraster), 
+           function(thislayer){
+             rast(myraster[[thislayer]])
+           })
+  )
+  
+}
