@@ -1,4 +1,4 @@
-###################################################################################################
+# BIOMOD_Tuning Documentation -------------------------------------------------
 ##' @name BIOMOD_Tuning
 ##' @author Frank Breiner
 ##' 
@@ -13,7 +13,7 @@
 ##' \code{\link{BIOMOD_ModelingOptions}} function
 ##' @param models a \code{vector} containing model names to be tuned, \cr 
 ##' must be among \code{GLM}, \code{GBM}, \code{GAM}, \code{CTA}, \code{ANN}, \code{SRE}, 
-##' \code{FDA}, \code{MARS}, \code{RF}, \code{MAXENT.Phillips}
+##' \code{FDA}, \code{MARS}, \code{RF}, \code{MAXENT}
 ##' @param metric.eval a \code{character} corresponding to the evaluation metric used to select 
 ##' optimal models and tune parameters, must be either \code{ROC} or \code{TSS} 
 ##' (\emph{maximizing Sensitivity and Specificity})
@@ -54,12 +54,12 @@
 ##' must be \code{earth} (see 
 ##' \url{http://topepo.github.io/caret/train-models-by-tag.html#Multivariate_Adaptive_Regression_Splines})
 ##' @param ME.metric a \code{character} corresponding to the evaluation metric used to select 
-##' optimal model and tune parameters for \code{MAXENT.Phillips}, must be either 
+##' optimal model and tune parameters for \code{MAXENT}, must be either 
 ##' \code{auc.val.avg}, \code{auc.diff.avg}, \code{or.mtp.avg}, \code{or.10p.avg} or \code{AICc}
 ##' @param ME.cvmethod a \code{character} corresponding to the method used to partition data for 
-##' \code{MAXENT.Phillips}, \cr must be \code{randomkfold}
+##' \code{MAXENT}, \cr must be \code{randomkfold}
 ##' @param ME.kfolds an \code{integer} corresponding to the number of bins for k-fold 
-##' cross-validation for \code{MAXENT.Phillips}
+##' cross-validation for \code{MAXENT}
 ##' @param ME.overlap (\emph{optional, default} \code{FALSE}) \cr 
 ##' A \code{logical} value defining whether to calculate pairwise metric of niche overlap or not 
 ##' (see \code{\link[ENMeval]{calc.niche.overlap}})
@@ -67,13 +67,14 @@
 ##' A \code{logical} value defining whether \emph{Features are constrained to remain within the 
 ##' range of values in the training data} (Elith et al. 2011) or not
 ##' @param ME.n.bg an \code{integer} corresponding to the number of background points used to run 
-##' \code{MAXENT.Phillips}
-##' @param ME.env a \code{RasterStack} object containing model predictor variables
+##' \code{MAXENT}
+##' @param ME.env a \code{\link[terra:rast]{SpatRaster}} object 
+##' containing model predictor variables
 ##' @param ME.parallel (\emph{optional, default} \code{TRUE}) \cr 
 ##' A \code{logical} value defining whether to enable parallel computing for 
-##' \code{MAXENT.Phillips} or not
+##' \code{MAXENT} or not
 ##' @param ME.numCores an \code{integer} corresponding to the number of cores to be used to 
-##' train \code{MAXENT.Phillips}
+##' train \code{MAXENT}
 ##' @param RF.method a \code{character} corresponding to the classification or regression model 
 ##' to use for \code{RF}, \cr 
 ##' must be \code{rf} (see \url{http://topepo.github.io/caret/train-models-by-tag.html#random-forest})
@@ -93,7 +94,7 @@
 ##'   \code{caret::trainControl(method = 'cv', summaryFunction = caret::twoClassSummary,} \cr
 ##'   \code{classProbs = TRUE, returnData = FALSE)}.
 ##'   \item All control parameters for other models are set to \code{ctrl.train} if unspecified.
-##'   \item For more details on \code{MAXENT.Phillips} tuning, please refer to 
+##'   \item For more details on \code{MAXENT} tuning, please refer to 
 ##'   \code{\link[ENMeval]{ENMevaluate}}.
 ##'   \item For more details on other models tuning, please refer to \code{\link[caret]{train}}.
 ##' }
@@ -118,10 +119,10 @@
 ##' 
 ##' 
 ##' @examples
+##' library(terra)
 ##' 
 ##' # Load species occurrences (6 species available)
-##' myFile <- system.file('external/species/mammals_table.csv', package = 'biomod2')
-##' DataSpecies <- read.csv(myFile, row.names = 1)
+##' data(DataSpecies)
 ##' head(DataSpecies)
 ##' 
 ##' # Select the name of the studied species
@@ -134,15 +135,15 @@
 ##' myRespXY <- DataSpecies[, c('X_WGS84', 'Y_WGS84')]
 ##' 
 ##' # Load environmental variables extracted from BIOCLIM (bio_3, bio_4, bio_7, bio_11 & bio_12)
-##' myFiles <- paste0('external/bioclim/current/bio', c(3, 4, 7, 11, 12), '.grd')
-##' myExpl <- raster::stack(system.file(myFiles, package = 'biomod2'))
+##' data(bioclim_current)
+##' myExpl <- terra::rast(bioclim_current)
 ##' 
 ##' \dontshow{
-##' myExtent <- raster::extent(0,30,45,70)
-##' myExpl <- raster::stack(raster::crop(myExpl, myExtent))
+##' myExtent <- terra::ext(0,30,45,70)
+##' myExpl <- terra::crop(myExpl, myExtent)
 ##' }
 ##' 
-##' # ---------------------------------------------------------------
+##' # --------------------------------------------------------------- #
 ##' # Format Data with true absences
 ##' myBiomodData <- BIOMOD_FormatingData(resp.var = myResp,
 ##'                                      expl.var = myExpl,
@@ -150,7 +151,7 @@
 ##'                                      resp.name = myRespName)
 ##' 
 ##' 
-##' # ---------------------------------------------------------------
+##' # --------------------------------------------------------------- #
 ##' ### Duration for turing all models sequential with default settings 
 ##' ### on 3.4 GHz processor: approx. 45 min tuning all models in parallel
 ##' ### (on 8 cores) using foreach loops runs much faster: approx. 14 min
@@ -191,12 +192,12 @@
 ##' 
 ##' @export
 ##' 
-###################################################################################################
+#------------------------------------------------------------------------------#
 
 
 BIOMOD_Tuning <- function(bm.format,
                           bm.options = BIOMOD_ModelingOptions(),
-                          models = c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE', 'FDA', 'MARS', 'RF', 'MAXENT.Phillips'),
+                          models = c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE', 'FDA', 'MARS', 'RF', 'MAXENT'),
                           metric.eval = 'ROC',
                           ctrl.train = NULL,
                           ctrl.train.tuneLength = 30,
@@ -235,12 +236,17 @@ BIOMOD_Tuning <- function(bm.format,
   ## MAXENT: http://cran.r-project.org/web/packages/ENMeval/ENMeval.pdf --> ENMevaluate()
   ## or:    http://cran.r-project.org/web/packages/maxent/maxent.pdf -->  tune.maxent()
   
-  ## 0. Check namespaces --------------------------------------------------------------------------
-  mod.names = c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE', 'FDA', 'MARS', 'RF', 'MAXENT.Phillips')
+  ## 0. Check namespaces ------------------------------------------------------
+  
+  mod.names = c('GLM', 'GBM', 'GAM', 'CTA', 'ANN', 'SRE', 'FDA', 'MARS', 'RF', 'MAXENT')
   
   if (sum(mod.names %in% models) > 0) {
-    if (!isNamespaceLoaded("caret")) { requireNamespace("caret", quietly = TRUE) }
-    if (!isNamespaceLoaded('dplyr')) { requireNamespace("dplyr", quietly = TRUE) }
+    if (!isNamespaceLoaded("caret")) { 
+      if(!requireNamespace('caret', quietly = TRUE)) stop("Package 'caret' not found")
+    }
+    if (!isNamespaceLoaded('dplyr')) { 
+      if(!requireNamespace('dplyr', quietly = TRUE)) stop("Package 'dplyr' not found")
+    }
     if (is.null(ctrl.train)) {
       ctrl.train <- caret::trainControl(method = "cv",
                                         repeats = 3,
@@ -248,12 +254,16 @@ BIOMOD_Tuning <- function(bm.format,
                                         classProbs = TRUE,
                                         returnData = FALSE)
     }
-    if ("MAXENT.Phillips" %in% models && !isNamespaceLoaded('ENMeval')) { requireNamespace("ENMeval", quietly = TRUE) }
+    if ("MAXENT" %in% models && !isNamespaceLoaded('ENMeval')) { 
+      if(!requireNamespace('ENMeval', quietly = TRUE)) stop("Package 'ENMeval' not found")
+    }
     # if ("MAXENT.Tsuruoka" %in% models && !isNamespaceLoaded('maxent')) { requireNamespace("maxent", quietly = TRUE) }
-    if ("SRE" %in% models && !isNamespaceLoaded('dismo')) { requireNamespace("dismo", quietly = TRUE) }
+    if ("SRE" %in% models && !isNamespaceLoaded('dismo')) { 
+      if(!requireNamespace('dismo', quietly = TRUE)) stop("Package 'dismo' not found")
+    }
   }
   
-  tune.SRE <- tune.GLM <- tune.MAXENT.Phillips <- tune.GAM <- tune.GBM <- 
+  tune.SRE <- tune.GLM <- tune.MAXENT <- tune.GAM <- tune.GBM <- 
     tune.CTA.rpart <- tune.CTA.rpart2 <- tune.RF <- tune.ANN <- tune.MARS <- tune.FDA <- NULL
   # tune.MAXENT.Tsuruoka <- NULL
   
@@ -261,7 +271,7 @@ BIOMOD_Tuning <- function(bm.format,
   # if (is.null(weights)) { weights = rep(1, length(bm.format@data.species))}
   
   
-  ## 1.1 SRE --------------------------------------------------------------------------------------
+  ## 1.1 SRE ------------------------------------------------------------------
   
   if ('SRE' %in% models)
   {
@@ -299,7 +309,7 @@ BIOMOD_Tuning <- function(bm.format,
   
   if(metric.eval == 'ROC' | metric.eval == 'TSS'){ resp <- as.factor(ifelse(resp == 1 & !is.na(resp), "Presence", "Absence")) }
   
-  ## 1.2 GBM --------------------------------------------------------------------------------------
+  ## 1.2 GBM ------------------------------------------------------------------
   
   if ('GBM' %in% models)
   {  
@@ -369,7 +379,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning GBM", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.3 RF ---------------------------------------------------------------------------------------
+  ## 1.3 RF -------------------------------------------------------------------
   
   if ('RF' %in% models)
   {
@@ -398,7 +408,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning RF", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.4 ANN --------------------------------------------------------------------------------------
+  ## 1.4 ANN -------------------------------------------------------------------
   
   if ('ANN' %in% models)
   {
@@ -445,7 +455,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning ANN", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.5 GAM --------------------------------------------------------------------------------------
+  ## 1.5 GAM ------------------------------------------------------------------
   
   if ('GAM' %in% models)
   {
@@ -473,7 +483,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning GAM", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.6 MARS -------------------------------------------------------------------------------------
+  ## 1.6 MARS ----------------------------------------------------------------
   
   if ('MARS' %in% models)
   {
@@ -517,13 +527,15 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning MARS", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.7 GLM --------------------------------------------------------------------------------------
+  ## 1.7 GLM ----------------------------------------------------------------
   
   if ('GLM' %in% models)
   {
     cat(paste("\n-=-=-=-=-=-=-=-=-=-=\n", "Start tuning GLM\n"))
     if (is.null(ctrl.GLM)) { ctrl.GLM <- ctrl.train }
-    if ("s_smoother" %in% GLM.type) { requireNamespace("gam", quietly = TRUE) }
+    if ("s_smoother" %in% GLM.type) { 
+      if(!requireNamespace('gam', quietly = TRUE)) stop("Package 'gam' not found")
+      }
     
     fm <- list()
     GLM.results = foreach (type = GLM.type, .combine = "rbind") %:%
@@ -552,7 +564,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning GLM", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }      
   
-  ## 1.8 FDA --------------------------------------------------------------------------------------
+  ## 1.8 FDA -----------------------------------------------------------------
   
   if ('FDA' %in% models)
   {
@@ -583,7 +595,7 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning FDA", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.9 CTA --------------------------------------------------------------------------------------
+  ## 1.9 CTA ------------------------------------------------------------------
   
   if ('CTA' %in% models)
   {
@@ -634,14 +646,14 @@ BIOMOD_Tuning <- function(bm.format,
     cat(paste("Finished tuning CTA", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
-  ## 1.10 MAXENT.Phillips -------------------------------------------------------------------------
+  ## 1.10 MAXENT ------------------------------------------------------
   
-  if ('MAXENT.Phillips' %in% models)
+  if ('MAXENT' %in% models)
   {
-    cat(paste("\n-=-=-=-=-=-=-=-=-=-=\n", "Start tuning MAXENT.Phillips\n"))
+    cat(paste("\n-=-=-=-=-=-=-=-=-=-=\n", "Start tuning MAXENT\n"))
     if (ME.cvmethod != 'randomkfold') { ME.kfolds <- NA }
     
-    try(tune.MAXENT.Phillips <- .maxent_tuning(pres = bm.format@data.env.var[bm.format@data.species == 1 & !is.na(bm.format@data.species), ],
+    try(tune.MAXENT <- .maxent_tuning(pres = bm.format@data.env.var[bm.format@data.species == 1 & !is.na(bm.format@data.species), ],
                                                bg = bm.format@data.env.var[bm.format@data.species == 0 | is.na(bm.format@data.species), ],
                                                method = ME.cvmethod, 
                                                kfolds = ME.kfolds, #ME.env,
@@ -650,27 +662,27 @@ BIOMOD_Tuning <- function(bm.format,
                                                numCores = ME.numCores,
                                                categoricals = NULL))
     
-    if (!is.null(tune.MAXENT.Phillips)) {
+    if (!is.null(tune.MAXENT)) {
       if (!ME.metric %in% c("auc.val.avg", "auc.diff.avg", "or.mtp.avg", "or.10p.avg", "AICc")) {
         ME.metric <- "auc.val.avg"
         cat("Invalid ME.metric argument! ME.metric was set to auc.val.avg")
       }
       if (ME.metric == 'auc.val.avg') {
-        tmp = which.max(tune.MAXENT.Phillips@results[, ME.metric])
+        tmp = which.max(tune.MAXENT@results[, ME.metric])
       } else {
-        tmp = which.min(tune.MAXENT.Phillips@results[, ME.metric])
+        tmp = which.min(tune.MAXENT@results[, ME.metric])
       }
-      bm.options@MAXENT.Phillips$linear <- grepl("L", tune.MAXENT.Phillips@results[tmp, "fc"])
-      bm.options@MAXENT.Phillips$quadratic <- grepl("Q", tune.MAXENT.Phillips@results[tmp, "fc"])
-      bm.options@MAXENT.Phillips$hinge <- grepl("H", tune.MAXENT.Phillips@results[tmp, "fc"])
-      bm.options@MAXENT.Phillips$product <- grepl("P", tune.MAXENT.Phillips@results[tmp, "fc"])
-      bm.options@MAXENT.Phillips$threshold <- grepl("T", tune.MAXENT.Phillips@results[tmp, "fc"])
-      bm.options@MAXENT.Phillips$betamultiplier <- tune.MAXENT.Phillips@results[tmp, "rm"]
+      bm.options@MAXENT$linear <- grepl("L", tune.MAXENT@results[tmp, "fc"])
+      bm.options@MAXENT$quadratic <- grepl("Q", tune.MAXENT@results[tmp, "fc"])
+      bm.options@MAXENT$hinge <- grepl("H", tune.MAXENT@results[tmp, "fc"])
+      bm.options@MAXENT$product <- grepl("P", tune.MAXENT@results[tmp, "fc"])
+      bm.options@MAXENT$threshold <- grepl("T", tune.MAXENT@results[tmp, "fc"])
+      bm.options@MAXENT$betamultiplier <- tune.MAXENT@results[tmp, "rm"]
     } else {
-      cat("Tuning MAXENT.Phillips failed!")
-      tune.MAXENT.Phillips <- "FAILED"
+      cat("Tuning MAXENT failed!")
+      tune.MAXENT <- "FAILED"
     }
-    cat(paste("Finished tuning MAXENT.Phillips", "\n-=-=-=-=-=-=-=-=-=-=\n"))
+    cat(paste("Finished tuning MAXENT", "\n-=-=-=-=-=-=-=-=-=-=\n"))
   }
   
   
@@ -690,13 +702,13 @@ BIOMOD_Tuning <- function(bm.format,
   .bm_cat("Done")
   return(list(models.options = bm.options, tune.SRE = tune.SRE,  tune.CTA.rpart = tune.CTA.rpart, tune.CTA.rpart2 = tune.CTA.rpart2,
               tune.RF = tune.RF, tune.ANN = tune.ANN,  tune.MARS = tune.MARS, tune.FDA = tune.FDA, tune.GBM = tune.GBM,
-              tune.GAM = tune.GAM, tune.GLM = tune.GLM, tune.MAXENT.Phillips = tune.MAXENT.Phillips))
+              tune.GAM = tune.GAM, tune.GLM = tune.GLM, tune.MAXENT = tune.MAXENT))
   # tune.MAXENT.Tsuruoka = tune.MAXENT.Tsuruoka, 
 }
 
 
-###################################################################################################
-#### Modified tuning function from the ENMeval package to tune MAXENT.Phillips (internal function for BIOMOD_tuning)
+## Maxent Tuning ---------------------------------------------------------------
+#### Modified tuning function from the ENMeval package to tune MAXENT (internal function for BIOMOD_tuning)
 
 .maxent_tuning <- function(pres,
                            bg,
