@@ -1,4 +1,4 @@
-###################################################################################################
+# bm_PseudoAbsences doc ---------------------------------------------------------
 ##' @name bm_PseudoAbsences
 ##' @aliases bm_PseudoAbsences
 ##' @aliases bm_PseudoAbsences_random
@@ -18,7 +18,7 @@
 ##' \code{1} : presence, \code{NA} : indeterminate) for a single species that will be used to 
 ##' find the pseudo-absences
 ##' @param expl.var a \code{matrix}, \code{data.frame}, \code{\link[sp]{SpatialPointsDataFrame}} 
-##' or \code{\link[raster:stack]{RasterStack}} object containing the explanatory variables (in 
+##' or \code{\link[terra:rast]{SpatRaster}} object containing the explanatory variables (in 
 ##' columns or layers) that will be used to find the pseudo-absences
 ##' @param \ldots (\emph{optional, one or several of the following arguments depending on the selected 
 ##' method)}) 
@@ -93,14 +93,13 @@
 ##' @family Secundary functions
 ##'
 ##' 
-##' @importFrom raster extract coordinates subset reclassify mask cellFromXY xyFromCell distance 
-##' sampleRandom
-##' @importFrom sp SpatialPointsDataFrame coordinates
-##' 
+##' @importFrom terra rast vect freq spatSample values extract
+##' @importFrom utils packageVersion
+##'
 ##' @export
 ##' 
 ##' 
-###################################################################################################
+## --------------------------------------------------------------------------- #
 
 
 bm_PseudoAbsences <- function(resp.var, expl.var, nb.rep = 1, strategy = 'random', dist.min = 0, dist.max = NULL
@@ -126,30 +125,40 @@ bm_PseudoAbsences <- function(resp.var, expl.var, nb.rep = 1, strategy = 'random
 }
 
 
-###################################################################################################
+# Argument Check --------------------------------------------------------------
 
 .bm_PseudoAbsences.check.args <- function(resp.var, expl.var, nb.rep, strategy, dist.min, dist.max, nb.absences, sre.quant)
 {
   cat('\n\nChecking Pseudo-absence selection arguments...\n')
-  
   ## 1. Check resp.var argument -----------------------------------------------
   if (is.vector(resp.var)) {
-    resp.var <- SpatialPointsDataFrame(matrix(0, ncol = 2, nrow = length(resp.var)), data.frame(resp.var))
+    resp.var <- vect(data.frame(x = 0,
+                                y = 0,
+                                resp = resp.var),
+                     geom = c("x","y"))
   }
-  .fun_testIfInherits(TRUE, "resp.var", resp.var, "SpatialPoints")
+  .fun_testIfInherits(TRUE, "resp.var", resp.var, "SpatVector")
   
   ## 2. Check expl.var argument -----------------------------------------------
-  if (is.matrix(expl.var) | is.data.frame(expl.var)) {
+  if (inherits(expl.var, c("matrix","data.frame"))) {
     if (nrow(expl.var) != nrow(resp.var)) {
       stop("Species and Explanatory must have same dimensions")
     }
-    expl.var <- SpatialPointsDataFrame(coordinates(resp.var), as.data.frame(expl.var))
+    # transform expl.var into SpatVector
+    rownames(expl.var) <- NULL
+    expl.var <- vect(
+      cbind(data.frame(x = crds(resp.var)[,1],
+                       y = crds(resp.var)[,2]),
+            as.data.frame(expl.var)
+      ),
+      geom = c("x","y")
+    )
   }
-  .fun_testIfInherits(TRUE, "expl.var", expl.var, c("SpatialPoints", "Raster"))
+  .fun_testIfInherits(TRUE, "expl.var", expl.var, c("SpatVector", "SpatRaster"))
   
   ## 3. Check strategy argument -----------------------------------------------
   availableStrategies <- c("random", "sre", "disk", "user.defined")
-  if (!(strategy %in% availableStrategies) || sum(abs(coordinates(resp.var))) == 0) {
+  if (!(strategy %in% availableStrategies) || all(crds(resp.var) == 0)) {
     # no coordinates or unknown strategy
     strategy <- "random"
     cat("\n   ! Random strategy was automatically selected (that can be due to points coordinates lack or unavailable strategy choosen)")
@@ -196,27 +205,37 @@ bm_PseudoAbsences <- function(resp.var, expl.var, nb.rep = 1, strategy = 'random
               sre.quant = sre.quant))
 }
 
-###################################################################################################
+# Additionnal tools ------------------------------------------------------------
 
 .get_nb_true_abs <- function(sp)
 {
   if (is.vector(sp)) {
-    return(sum(sp == 0, na.rm = TRUE))
-  } else if (inherits(sp, 'SpatialPoints')) {
-    return(sum(sp@data == 0, na.rm = TRUE))
-  } else if (inherits(sp, 'Raster')) {
-    return(sum(sp[] == 0, na.rm = TRUE))
+    return(length(which(sp == 0)))
+  } else if (inherits(sp, 'SpatVector')) {
+    return(length(which(values(sp)[,1] == 0)))
+  } else if (inherits(sp, 'SpatRaster')) {
+    return(freq(sp, value = 0)$count)
   }
 }
 
 .get_nb_available_pa_cells <- function(data, PA.flag = NA)
 {
-  if (is.vector(data) || is.data.frame(data) || is.matrix(data)) {
-    return(ifelse(is.na(PA.flag), sum(is.na(data)), sum(data == PA.flag, na.rm = TRUE)))
-  } else if(inherits(data, 'SpatialPoints')){
-    return(ifelse(is.na(PA.flag), sum(is.na(data@data)), sum(data@data == PA.flag, na.rm = TRUE)))
-  } else if(inherits(data, 'Raster')){
-    return(ifelse(is.na(PA.flag), sum(is.na(data[])), sum(data[] == PA.flag, na.rm = TRUE)))
+  if (is.vector(data) || inherits(data, c("matrix","data.frame"))) {
+    return(
+      ifelse(is.na(PA.flag),
+             length(which(is.na(data))), 
+             length(which(data == PA.flag)))
+    )
+  } else if (inherits(data, 'SpatVector')) {
+    return(
+      ifelse(is.na(PA.flag), 
+             length(which(is.na(as.data.frame(data)[,1]))), 
+             length(which(values(data)[,1] == PA.flag)))
+    )
+  } else if (inherits(data, 'SpatRaster')) {
+    return(
+      freq(data, value = PA.flag)$count
+    )
   }
 }
 
@@ -227,7 +246,7 @@ bm_PseudoAbsences <- function(resp.var, expl.var, nb.rep = 1, strategy = 'random
   #   rn = rep("", nrow(xy))
   # }
   missing_rn <- which(rn == "")
-  if (length(missing_rn)) {
+  if (length(missing_rn) > 0) {
     rn[missing_rn] <- paste0("pa", 1:length(missing_rn))
   }
   rownames(xy) <- rn
@@ -235,7 +254,7 @@ bm_PseudoAbsences <- function(resp.var, expl.var, nb.rep = 1, strategy = 'random
 }
 
 
-###################################################################################################
+# bm_PseudoAbsences user-defined methods --------------------------------------
 
 ##'
 ##' @rdname bm_PseudoAbsences
@@ -247,37 +266,40 @@ setGeneric("bm_PseudoAbsences_user.defined",
              standardGeneric( "bm_PseudoAbsences_user.defined")
            })
 
+## bm_PseudoAbsences user-defined SpatVector methods --------------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
 ##'
 
-setMethod('bm_PseudoAbsences_user.defined', signature(expl.var = "SpatialPointsDataFrame"),
+setMethod('bm_PseudoAbsences_user.defined', signature(expl.var = "SpatVector"),
           function(resp.var, expl.var, user.table) {
             cat("\n   > User defined pseudo absences selection")
-            return(list(xy = coordinates(resp.var),
-                        sp = as.numeric(unlist(resp.var@data, use.names = FALSE)),
-                        env = as.data.frame(expl.var@data),
-                        pa.tab = user.table))
-          })
-
-##'
-##' @rdname bm_PseudoAbsences
-##' @export
-##'
-
-setMethod('bm_PseudoAbsences_user.defined', signature(expl.var = "RasterStack"), 
-          function(resp.var, expl.var, user.table) {
-            cat("\n   > User defined pseudo absences selection")
-            expl.var <- as.data.frame(extract(expl.var, coordinates(resp.var)))
-            return(list(xy = coordinates(resp.var),
-                        sp = as.numeric(unlist(resp.var@data, use.names = FALSE)), 
+            return(list(xy = crds(resp.var),
+                        sp = as.numeric(unlist(values(resp.var), use.names = FALSE)),
                         env = as.data.frame(expl.var),
                         pa.tab = user.table))
           })
 
+## bm_PseudoAbsences user-defined SpatRaster methods --------------------------------------
+##'
+##' @rdname bm_PseudoAbsences
+##' @export
+##'
+
+setMethod('bm_PseudoAbsences_user.defined', signature(expl.var = "SpatRaster"), 
+          function(resp.var, expl.var, user.table) {
+            cat("\n   > User defined pseudo absences selection")
+            expl.var <- extract(expl.var, resp.var, ID = FALSE)
+            return(list(xy = crds(resp.var),
+                        sp = as.numeric(unlist(values(resp.var), use.names = FALSE)), 
+                        env = expl.var,
+                        pa.tab = user.table))
+          })
+
+
+# bm_PseudoAbsences random methods --------------------------------------
 
-###################################################################################################
 
 ##'
 ##' @rdname bm_PseudoAbsences
@@ -289,12 +311,13 @@ setGeneric("bm_PseudoAbsences_random",
              standardGeneric( "bm_PseudoAbsences_random")
            })
 
+## bm_PseudoAbsences random SPDF methods --------------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
 ##'
 
-setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatialPointsDataFrame"),
+setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatVector"),
           function(resp.var, expl.var, nb.absences, nb.rep)
           {
             cat("\n   > random pseudo absences selection")
@@ -313,15 +336,16 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatialPointsDataFra
               # 3. Select always the presences and the true absences
               pa.tab <- matrix(FALSE, ncol = nb.rep, nrow = nrow(resp.var))
               colnames(pa.tab) <- paste0("PA", 1:nb.rep)
-              pa.tab[c(which(resp.var@data[, 1] == 1), which(resp.var@data[, 1] == 0)),] <- TRUE
+              pa.tab[c(which(values(resp.var)[, 1] == 1),
+                       which(values(resp.var)[, 1] == 0)),] <- TRUE
               
               # 4. For each repetition, select among NA cells
-              cand.cells <- which(is.na(resp.var@data[, 1]))
+              cand.cells <- which(is.na(values(resp.var)[, 1]))
               for (j in 1:ncol(pa.tab)) {
                 ## force to get at least one value of each factorial variable
                 fact.level.cells <- bm_SampleFactorLevels(expl.var = as.data.frame(expl.var),
                                                           mask.out = pa.tab[, j, drop = FALSE])
-                if (length(fact.level.cells)) {
+                if (length(fact.level.cells) > 0) {
                   pa.tab[fact.level.cells, j] <- TRUE
                   cand.cells <- setdiff(cand.cells, fact.level.cells)
                 }
@@ -330,9 +354,9 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatialPointsDataFra
                               replace = FALSE), j] <- TRUE
               }
               
-              return(list(xy = coordinates(resp.var),
-                          sp = as.numeric(unlist(resp.var@data, use.names = FALSE)),
-                          env = as.data.frame(expl.var@data),
+              return(list(xy = crds(resp.var),
+                          sp = as.numeric(unlist(values(resp.var), use.names = FALSE)),
+                          env = as.data.frame(expl.var),
                           pa.tab = pa.tab))
             } else {
               cat("\nUnsupported case yet!")
@@ -340,16 +364,17 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatialPointsDataFra
             }
           })
 
+## bm_PseudoAbsences random SpatRaster methods --------------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
+##' @importFrom terra rast xyFromCell cellFromXY classify crds extract subset
 ##'
 
-setMethod('bm_PseudoAbsences_random', signature(expl.var = "RasterStack"),
+setMethod('bm_PseudoAbsences_random', signature(expl.var = "SpatRaster"),
           function(resp.var, expl.var, nb.absences, nb.rep)
           {
             cat("\n   > random pseudo absences selection")
-            
             # 1. Check if NA are present in resp.var observations or not to determine which dataset to use
             nb.cells <- .get_nb_available_pa_cells(resp.var)
             if (nb.cells > 0) { # PA will be taken into response variable
@@ -364,29 +389,31 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "RasterStack"),
               # 3. Select always the presences and the true absences
               pa.tab <- matrix(FALSE, ncol = nb.rep, nrow = nrow(resp.var))
               colnames(pa.tab) <- paste0("PA", 1:nb.rep)
-              pa.tab[c(which(resp.var@data[, 1] == 1), which(resp.var@data[, 1] == 0)),] <- TRUE
+              pa.tab[c(which(values(resp.var)[, 1] == 1),
+                       which(values(resp.var)[, 1] == 0)),] <- TRUE
               
               # 4. For each repetition, select among NA cells
-              cand.cells <- which(is.na(resp.var@data[, 1]))
+              cand.cells <- which(is.na(values(resp.var)[, 1]))
               for (j in 1:ncol(pa.tab)) {
                 pa.tab[sample(x = cand.cells, size = nb.absences, replace = FALSE), j] <- TRUE
               }
               
-              expl.var <- as.data.frame(extract(expl.var, coordinates(resp.var)))
-              return(list(xy = coordinates(resp.var),
-                          sp = as.numeric(unlist(resp.var@data, use.names = FALSE)), 
+              expl.var <- extract(expl.var, resp.var, ID = FALSE)
+              return(list(xy = crds(resp.var),
+                          sp = as.numeric(unlist(values(resp.var), use.names = FALSE)), 
                           env = as.data.frame(expl.var),
                           pa.tab = as.data.frame(pa.tab)))
             } else {
               cat("\n   > Pseudo absences are selected in explanatory variables")
-              
               # create a mask containing all not already sampled points (presences and absences)
-              mask.env <- mask.out <- subset(expl.var, 1, drop = TRUE)
-              mask.env <- reclassify(mask.env, c(-Inf, Inf, -1)) ## the area we want to sample
+              mask.env <- mask.out <- subset(expl.var, 1)
+              ## the area we want to sample
+              mask.env <- classify(mask.env, matrix(c(-Inf, Inf, -1), 
+                                                    ncol = 3, byrow = TRUE)) 
               mask.out[] <- NA
               
               # add presences and true absences in our mask
-              in.cells <- cellFromXY(mask.env, coordinates(resp.var))
+              in.cells <- cellFromXY(mask.env, crds(resp.var))
               mask.env[in.cells] <- NA
               mask.out[in.cells] <- 1
               
@@ -412,22 +439,19 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "RasterStack"),
                   
                   ## force to get at least one value of each factorial variable
                   fact.level.cells <- bm_SampleFactorLevels(expl.var = expl.var, mask.out = mask.out)
-                  if (length(fact.level.cells)) {
+                  if (length(fact.level.cells) > 0) {
                     SR <- c(SR, fact.level.cells)
                     mask.env.tmp[SR] <- NA ## update the mask by removing already selected cells
                   }
-                  
-                  SR <- c(SR, sampleRandom(x = mask.env.tmp,
+                  ## repeat sampling until having the right number of points
+                  ## spatSample with na.rm = TRUE may return less points than asked
+                  while(length(SR) < nb.absences){
+                    SR <- c(SR, spatSample(x = mask.env.tmp,
+                                           method = "random",
                                            size = nb.absences - length(SR),
                                            cells = TRUE,
                                            na.rm = TRUE)[, "cell", drop = TRUE])
-                  ## repeat sampling until having the right number of points
-                  while(length(SR)<nb.absences){
                     mask.env.tmp[SR] <- NA ## update the mask by removing already selected cells
-                    SR <- c(SR, sampleRandom(x = mask.env.tmp,
-                                             size = nb.absences - length(SR),
-                                             cells = TRUE,
-                                             na.rm = TRUE)[, "cell", drop = TRUE])
                   }
                   pa.tab.tmp[, j] <- SR
                 }
@@ -441,14 +465,18 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "RasterStack"),
                 }
                 
                 # putting presences, true absences and pseudo absences together
-                xy = coordinates(resp.var)
+                xy = crds(resp.var)
                 rownames(xy) = paste0("pres", 1:nrow(xy))
                 xy <- rbind(xy, xyFromCell(mask.env, selected.cells))
                 xy <- .add_pa_rownames(xy)
-                resp.var <- as.numeric(unlist(c(resp.var@data[, 1], rep(NA, length(selected.cells))), use.names = FALSE))
+                resp.var <- as.numeric(unlist(c(values(resp.var)[, 1], 
+                                                rep(NA, length(selected.cells))),
+                                              use.names = FALSE))
                 expl.var <- extract(expl.var, xy)
-                pa.tab <- rbind(matrix(TRUE, nrow = (nrow(xy) - length(selected.cells)),
-                                       ncol = ncol(pa.tab)), pa.tab)
+                pa.tab <- rbind(matrix(TRUE, 
+                                       nrow = (nrow(xy) - length(selected.cells)),
+                                       ncol = ncol(pa.tab)), 
+                                pa.tab)
                 
                 return(list(xy = xy,
                             sp = resp.var,
@@ -459,7 +487,8 @@ setMethod('bm_PseudoAbsences_random', signature(expl.var = "RasterStack"),
           })
 
 
-###################################################################################################
+# bm_PseudoAbsences SRE methods --------------------------------------
+
 
 ##'
 ##' @rdname bm_PseudoAbsences
@@ -471,18 +500,19 @@ setGeneric("bm_PseudoAbsences_sre",
              standardGeneric("bm_PseudoAbsences_sre")
            })
 
+## bm_PseudoAbsences SRE SpatVector methods ----------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
 ##'
 
-setMethod('bm_PseudoAbsences_sre', signature(expl.var = "SpatialPointsDataFrame"), 
+setMethod('bm_PseudoAbsences_sre', signature(expl.var = "SpatVector"), 
           function(resp.var, expl.var, sre.quant, nb.absences, nb.rep)
           {
             cat("\n   > SRE pseudo absences selection")
             
             # 0. calculate SRE to determine available pixels
-            mask.in <- bm_SRE(resp.var = resp.var, expl.var = expl.var, new.env = expl.var@data, quant = sre.quant)
+            mask.in <- bm_SRE(resp.var = resp.var, expl.var = expl.var, new.env = values(expl.var), quant = sre.quant)
             mask.in <- data.frame(mask.in = !as.logical(mask.in)) ## revert the mask to sample PA out of SRE
             
             # 1. Check if NA are present in resp.var observations or not to determine which dataset to use
@@ -498,7 +528,7 @@ setMethod('bm_PseudoAbsences_sre', signature(expl.var = "SpatialPointsDataFrame"
             # 3. Select always the presences and the true absences
             pa.tab <- matrix(FALSE, ncol = nb.rep, nrow = nrow(resp.var))
             colnames(pa.tab) <- paste0("PA", 1:nb.rep)
-            pa.tab[c(which(resp.var@data[, 1] == 1), which(resp.var@data[, 1] == 0)),] <- TRUE
+            pa.tab[c(which(values(resp.var)[, 1] == 1), which(values(resp.var)[, 1] == 0)),] <- TRUE
             
             # 4. For each repetition, select among NA cells
             cand.cells <- which(!mask.in$mask.in)
@@ -513,18 +543,19 @@ setMethod('bm_PseudoAbsences_sre', signature(expl.var = "SpatialPointsDataFrame"
                               replace = FALSE)), j] <- TRUE
             }
             
-            return(list(xy = coordinates(resp.var),
-                        sp = as.numeric(unlist(resp.var@data, use.names = FALSE)),
-                        env = as.data.frame(expl.var@data),
+            return(list(xy = crds(resp.var),
+                        sp = as.numeric(unlist(values(resp.var), use.names = FALSE)),
+                        env = values(expl.var),
                         pa.tab = pa.tab))
           })
 
+## bm_PseudoAbsences SRE SpatRaster methods -----------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
 ##'
 
-setMethod('bm_PseudoAbsences_sre', signature(expl.var = "RasterStack"), 
+setMethod('bm_PseudoAbsences_sre', signature(expl.var = "SpatRaster"), 
           function(resp.var, expl.var, sre.quant, nb.absences, nb.rep)
           {
             cat("\n   > SRE pseudo absences selection")
@@ -555,22 +586,19 @@ setMethod('bm_PseudoAbsences_sre', signature(expl.var = "RasterStack"),
               
               ## force to get at least one value of each factorial variable
               fact.level.cells <- bm_SampleFactorLevels(expl.var = expl.var, mask.out = mask.out, mask.in = mask.in)
-              if (length(fact.level.cells)) {
+              if (length(fact.level.cells) > 0) {
                 SR <- c(SR, fact.level.cells)
                 mask.in.tmp[SR] <- NA ## update the mask by removing already selected cells
               }
               
-              SR <- c(SR, sampleRandom(x = mask.in.tmp,
+              ## repeat sampling until having the right number of points
+              while (length(SR) < nb.absences) {
+                SR <- c(SR, spatSample(x = mask.in.tmp,
+                                       method = "random",
                                        size = nb.absences - length(SR),
                                        cells = TRUE,
                                        na.rm = TRUE)[, "cell", drop = TRUE])
-              ## repeat sampling until having the right number of points
-              while (length(SR) < nb.absences) {
                 mask.in.tmp[SR] <- NA ## update the mask by removing already selected cells
-                SR <- c(SR, sampleRandom(x = mask.in.tmp,
-                                         size = nb.absences - length(SR),
-                                         cells = TRUE,
-                                         na.rm = TRUE)[, "cell", drop = TRUE])
               }
               pa.tab.tmp[, j] <- SR
             }
@@ -584,10 +612,12 @@ setMethod('bm_PseudoAbsences_sre', signature(expl.var = "RasterStack"),
             }
             
             # putting presences, true absences and pseudo absences together
-            xy <- rbind(coordinates(resp.var)[which(!is.na(resp.var@data[, 1])), ],
+            xy <- rbind(crds(resp.var)[which(!is.na(values(resp.var)[, 1])), ],
                         xyFromCell(mask.in, selected.cells))
             xy <- .add_pa_rownames(xy)
-            resp.var <- as.numeric(unlist(c(na.omit(resp.var@data[, 1]), rep(NA, length(selected.cells))), use.names = FALSE))
+            resp.var <- as.numeric(unlist(c(na.omit(values(resp.var)[, 1]),
+                                            rep(NA, length(selected.cells))),
+                                          use.names = FALSE))
             expl.var <- extract(expl.var, xy)
             pa.tab <- rbind(matrix(TRUE, nrow = (nrow(xy) - length(selected.cells)),
                                    ncol = ncol(pa.tab)), pa.tab)
@@ -599,7 +629,8 @@ setMethod('bm_PseudoAbsences_sre', signature(expl.var = "RasterStack"),
           })
 
 
-###################################################################################################
+# bm_PseudoAbsences disk methods --------------------------------------
+
 
 ##'
 ##' @rdname bm_PseudoAbsences
@@ -611,21 +642,20 @@ setGeneric("bm_PseudoAbsences_disk",
              standardGeneric("bm_PseudoAbsences_disk")
            })
 
+## bm_PseudoAbsences disk SpatVector methods --------------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
 ##'
 
-setMethod('bm_PseudoAbsences_disk', signature(expl.var = "SpatialPointsDataFrame"),
-          function(resp.var, expl.var, dist.min, dist.max, nb.absences, nb.rep)
-          {
+setMethod('bm_PseudoAbsences_disk', signature(expl.var = "SpatVector"),
+          function(resp.var, expl.var, dist.min, dist.max, nb.absences, nb.rep) {
             cat("\n   > Disk pseudo absences selection")
-            
             # 1. determining area which can be selected
-            coor <- coordinates(resp.var)
-            pres <- which(resp.var@data[, 1] == 1)
-            true.abs <- which(resp.var@data[, 1] == 0)
-            tmp.abs <- which(is.na(resp.var@data[, 1]))
+            coor <- crds(resp.var)
+            pres <- which(values(resp.var)[, 1] == 1)
+            true.abs <- which(values(resp.var)[, 1] == 0)
+            tmp.abs <- which(is.na(values(resp.var)[, 1]))
             outside <- rep(0, length(abs))
             inside <- rep(0, length(abs))
             
@@ -650,12 +680,14 @@ setMethod('bm_PseudoAbsences_disk', signature(expl.var = "SpatialPointsDataFrame
                                             nb.absences, nb.rep))
           })
 
+## bm_PseudoAbsences disk SpatRaster methods --------------------------------------
 ##'
 ##' @rdname bm_PseudoAbsences
 ##' @export
+##' @importFrom terra rast distance subset mask crds cellFromXY extract
 ##'
 
-setMethod('bm_PseudoAbsences_disk', signature(expl.var = "RasterStack"),
+setMethod('bm_PseudoAbsences_disk', signature(expl.var = "SpatRaster"),
           function(resp.var, expl.var, dist.min, dist.max, nb.absences, nb.rep)
           {
             cat("\n   > Disk pseudo absences selection")
@@ -663,28 +695,37 @@ setMethod('bm_PseudoAbsences_disk', signature(expl.var = "RasterStack"),
             # 1. Check if NA are present in resp.var observations or not to determine which dataset to use
             nb.cells <- .get_nb_available_pa_cells(resp.var)
             if (nb.cells > 0) { # PA will be taken into response variable
-              expl.var.tmp <- SpatialPointsDataFrame(coords = coordinates(resp.var),
-                                                     data = as.data.frame(extract(expl.var, coordinates(resp.var))))
-              return(bm_PseudoAbsences_disk(resp.var, expl.var.tmp, dist.min, dist.max, nb.absences, nb.rep))
+              expl.var.tmp <- extract(expl.var, resp.var[,-1], bind = TRUE, ID = FALSE)
+              return(
+                bm_PseudoAbsences_disk(resp.var, expl.var.tmp, 
+                                       dist.min, dist.max,
+                                       nb.absences, nb.rep)
+              )
             } else {
               cat("\n   > Pseudo absences are selected in explanatory variables")
               
               # create a mask
-              dist.mask <- subset(expl.var, 1, drop = TRUE)
+              dist.mask <- subset(expl.var, 1)
               dist.mask[] <- NA
               
-              pres.xy <- coordinates(resp.var[which(resp.var@data[, 1] == 1), ])
+              pres.xy <- crds(resp.var[which(values(resp.var)[, 1] == 1), ])
               dist.mask[cellFromXY(dist.mask, pres.xy)] <- 1
               
               dist.mask <- distance(dist.mask)
-              dist.mask <- mask(dist.mask, subset(expl.var, 1, drop = TRUE))
+              dist.mask <- mask(dist.mask, subset(expl.var, 1))
               
-              if (is.null(dist.max)) { dist.max <- Inf }
-              mask.in <- reclassify(dist.mask, c(-Inf, dist.min, NA, dist.min, dist.max, 1, dist.max, Inf, NA))
+              if (is.null(dist.max)) { 
+                dist.max <- Inf 
+                }
+              mask.in <- classify(dist.mask, 
+                                  matrix(c(-Inf, dist.min, NA,
+                                               dist.min, dist.max, 1, 
+                                               dist.max, Inf, NA),
+                                         ncol = 3, byrow = TRUE))
               mask.in[cellFromXY(mask.in, pres.xy)] <- 1
-              mask.in = stack(expl.var * mask.in)
+              mask.in = expl.var * mask.in
               names(mask.in) = names(expl.var)
-              
+
               # 2. selecting randomly pseudo absences
               return(bm_PseudoAbsences_random(resp.var, expl.var = mask.in, nb.absences, nb.rep))
             }
